@@ -14,13 +14,18 @@
  * limitations under the License.
  */
 
+/**
+ * @file ap_release.h
+ * @brief Version Release defines
+ */
+
 #ifndef AP_RELEASE_H
 #define AP_RELEASE_H
 
 #include "apr_general.h" /* stringify */
 
 #define AP_SERVER_COPYRIGHT \
-    "Copyright 2013 The Apache Software Foundation."
+    "Copyright 2007 The Apache Software Foundation."
 
 /*
  * The below defines the base string of the Server: header. Additional
@@ -35,24 +40,34 @@
  * Example: "Apache/1.1.0 MrWidget/0.1-alpha" 
  */
 #define AP_SERVER_BASEVENDOR "Apache Software Foundation"
+#define AP_SERVER_BASEPROJECT "Apache HTTP Server"
 #define AP_SERVER_BASEPRODUCT "Apache"
 
 #define AP_SERVER_MAJORVERSION_NUMBER 2
-#define AP_SERVER_MINORVERSION_NUMBER 0
-#define AP_SERVER_PATCHLEVEL_NUMBER   65
-#define AP_SERVER_ADD_STRING          ""
+#define AP_SERVER_MINORVERSION_NUMBER 2
+#define AP_SERVER_PATCHLEVEL_NUMBER   6
+#define AP_SERVER_DEVBUILD_BOOLEAN    0
 
-/* Synchronize the above with docs/manual/style/version.ent */ 
+#if AP_SERVER_DEVBUILD_BOOLEAN
+#define AP_SERVER_ADD_STRING          "-dev"
+#else
+#define AP_SERVER_ADD_STRING          ""
+#endif
 
 /* keep old macros as well */
-#define AP_SERVER_MAJORVERSION APR_STRINGIFY(AP_SERVER_MAJORVERSION_NUMBER)
-#define AP_SERVER_MINORVERSION APR_STRINGIFY(AP_SERVER_MINORVERSION_NUMBER)
-#define AP_SERVER_PATCHLEVEL APR_STRINGIFY(AP_SERVER_PATCHLEVEL_NUMBER) \
-                              AP_SERVER_ADD_STRING
+#define AP_SERVER_MAJORVERSION  APR_STRINGIFY(AP_SERVER_MAJORVERSION_NUMBER)
+#define AP_SERVER_MINORVERSION  APR_STRINGIFY(AP_SERVER_MINORVERSION_NUMBER)
+#define AP_SERVER_PATCHLEVEL    APR_STRINGIFY(AP_SERVER_PATCHLEVEL_NUMBER) \
+                                AP_SERVER_ADD_STRING
 
 #define AP_SERVER_MINORREVISION AP_SERVER_MAJORVERSION "." AP_SERVER_MINORVERSION
 #define AP_SERVER_BASEREVISION  AP_SERVER_MINORREVISION "." AP_SERVER_PATCHLEVEL
-#define AP_SERVER_BASEVERSION AP_SERVER_BASEPRODUCT "/" AP_SERVER_BASEREVISION
-#define AP_SERVER_VERSION  AP_SERVER_BASEVERSION
+#define AP_SERVER_BASEVERSION   AP_SERVER_BASEPRODUCT "/" AP_SERVER_BASEREVISION
+#define AP_SERVER_VERSION       AP_SERVER_BASEVERSION
+
+/* macro for Win32 .rc files using numeric csv representation */
+#define AP_SERVER_PATCHLEVEL_CSV AP_SERVER_MAJORVERSION_NUMBER ##, \
+                               ##AP_SERVER_MINORVERSION_NUMBER ##, \
+                               ##AP_SERVER_PATCHLEVEL_NUMBER
 
 #endif
