@@ -71,7 +71,6 @@
 #endif
 #define APR_WANT_STRFUNC
 #include "apr_want.h"
-#include "apr_strings.h"
 
 #ifdef NEXT
 #if (NX_CURRENT_COMPILER_RELEASE == 410)
@@ -99,7 +98,13 @@
 
 module AP_MODULE_DECLARE_DATA status_module;
 
-int server_limit, thread_limit;
+static int server_limit, thread_limit;
+
+/* Implement 'ap_run_status_hook'. */
+APR_IMPLEMENT_OPTIONAL_HOOK_RUN_ALL(ap, STATUS, int, status_hook,
+                                    (request_rec *r, int flags),
+                                    (r, flags),
+                                    OK, DECLINED)
 
 #ifdef HAVE_TIMES
 /* ugh... need to know if we're running with a pthread implementation
@@ -109,12 +114,6 @@ int server_limit, thread_limit;
 static pid_t child_pid;
 #endif
 
-/* Implement 'ap_run_status_hook'. */
-APR_IMPLEMENT_OPTIONAL_HOOK_RUN_ALL(ap, STATUS, int, status_hook,
-                                    (request_rec *r, int flags),
-                                    (r, flags),
-                                    OK, DECLINED)
-
 /*
  * command-related code. This is here to prevent use of ExtendedStatus
  * without status_module included.
@@ -230,10 +229,11 @@ static int status_handler(request_rec *r)
     worker_score *ws_record;
     process_score *ps_record;
     char *stat_buffer;
-    pid_t *pid_buffer;
+    pid_t *pid_buffer, worker_pid;
     clock_t tu, ts, tcu, tcs;
+    ap_generation_t worker_generation;
 
-    if (strcmp(r->handler, STATUS_MAGIC_TYPE) && 
+    if (strcmp(r->handler, STATUS_MAGIC_TYPE) &&
         strcmp(r->handler, "server-status")) {
         return DECLINED;
     }
@@ -282,18 +282,19 @@ static int status_handler(request_rec *r)
             if ((loc = ap_strstr_c(r->args,
                                    status_options[i].form_data_str)) != NULL) {
                 switch (status_options[i].id) {
-                case STAT_OPT_REFRESH: {
-                    apr_size_t len = strlen(status_options[i].form_data_str);
-                    long t = 0;
-
-                    if (*(loc + len ) == '=') {
-                        t = atol(loc + len + 1);
-                    }
-                    apr_table_set(r->headers_out,
-                                  status_options[i].hdr_out_str,
-                                  apr_ltoa(r->pool, t < 1 ? 10 : t));
+                case STAT_OPT_REFRESH:
+                    if (*(loc + strlen(status_options[i].form_data_str)) == '='
+                        && atol(loc + strlen(status_options[i].form_data_str)
+                                + 1) > 0)
+                        apr_table_set(r->headers_out,
+                                      status_options[i].hdr_out_str,
+                                      loc +
+                                      strlen(status_options[i].hdr_out_str) +
+                                      1);
+                    else
+                        apr_table_set(r->headers_out,
+                                      status_options[i].hdr_out_str, "1");
                     break;
-                }
                 case STAT_OPT_NOTABLE:
                     no_table_report = 1;
                     break;
@@ -313,7 +314,7 @@ static int status_handler(request_rec *r)
         clock_t proc_tu = 0, proc_ts = 0, proc_tcu = 0, proc_tcs = 0;
         clock_t tmp_tu, tmp_ts, tmp_tcu, tmp_tcs;
 #endif
-        
+
         ps_record = ap_get_scoreboard_process(i);
         for (j = 0; j < thread_limit; ++j) {
             int indx = (i * thread_limit) + j;
@@ -396,7 +397,7 @@ static int status_handler(request_rec *r)
         ap_rputs("<h1>Apache Server Status for ", r);
         ap_rvputs(r, ap_get_server_name(r), "</h1>\n\n", NULL);
         ap_rvputs(r, "<dl><dt>Server Version: ",
-                  ap_get_server_version(), "</dt>\n", NULL);
+                  ap_get_server_description(), "</dt>\n", NULL);
         ap_rvputs(r, "<dt>Server Built: ",
                   ap_get_server_built(), "\n</dt></dl><hr /><dl>\n", NULL);
         ap_rvputs(r, "<dt>Current Time: ",
@@ -573,7 +574,7 @@ static int status_handler(request_rec *r)
                     req_time = 0L;
                 else
                     req_time = (long)
-                        ((ws_record->stop_time - 
+                        ((ws_record->stop_time -
                           ws_record->start_time) / 1000);
                 if (req_time < 0L)
                     req_time = 0L;
@@ -584,21 +585,29 @@ static int status_handler(request_rec *r)
                 bytes = ws_record->bytes_served;
                 my_bytes = ws_record->my_bytes_served;
                 conn_bytes = ws_record->conn_bytes;
+                if (ws_record->pid) { /* MPM sets per-worker pid and generation */
+                    worker_pid = ws_record->pid;
+                    worker_generation = ws_record->generation;
+                }
+                else {
+                    worker_pid = ps_record->pid;
+                    worker_generation = ps_record->generation;
+                }
 
                 if (no_table_report) {
                     if (ws_record->status == SERVER_DEAD)
                         ap_rprintf(r,
                                    "<b>Server %d-%d</b> (-): %d|%lu|%lu [",
-                                   i, (int)ps_record->generation,
+                                   i, (int)worker_generation,
                                    (int)conn_lres, my_lres, lres);
                     else
                         ap_rprintf(r,
                                    "<b>Server %d-%d</b> (%"
                                    APR_PID_T_FMT "): %d|%lu|%lu [",
-                                   i, (int) ps_record->generation,
-                                   ps_record->pid,
+                                   i, (int) worker_generation,
+                                   worker_pid,
                                    (int)conn_lres, my_lres, lres);
-                    
+
                     switch (ws_record->status) {
                     case SERVER_READY:
                         ap_rputs("Ready", r);
@@ -637,7 +646,7 @@ static int status_handler(request_rec *r)
                         ap_rputs("?STATE?", r);
                         break;
                     }
-                    
+
                     ap_rprintf(r, "] "
 #ifdef HAVE_TIMES
                                "u%g s%g cu%g cs%g"
@@ -673,17 +682,18 @@ static int status_handler(request_rec *r)
                     if (ws_record->status == SERVER_DEAD)
                         ap_rprintf(r,
                                    "<tr><td><b>%d-%d</b></td><td>-</td><td>%d/%lu/%lu",
-                                   i, (int)ps_record->generation,
+                                   i, (int)worker_generation,
                                    (int)conn_lres, my_lres, lres);
                     else
                         ap_rprintf(r,
                                    "<tr><td><b>%d-%d</b></td><td>%"
                                    APR_PID_T_FMT
                                    "</td><td>%d/%lu/%lu",
-                                   i, (int)ps_record->generation,
-                                   ps_record->pid, (int)conn_lres,
+                                   i, (int)worker_generation,
+                                   worker_pid,
+                                   (int)conn_lres,
                                    my_lres, lres);
-                    
+
                     switch (ws_record->status) {
                     case SERVER_READY:
                         ap_rputs("</td><td>_", r);
@@ -722,7 +732,7 @@ static int status_handler(request_rec *r)
                         ap_rputs("</td><td>?", r);
                         break;
                     }
-                    
+
                     ap_rprintf(r,
                                "\n</td>"
 #ifdef HAVE_TIMES
@@ -738,11 +748,11 @@ static int status_handler(request_rec *r)
                                (long)apr_time_sec(nowtime -
                                                   ws_record->last_used),
                                (long)req_time);
-                    
+
                     ap_rprintf(r, "</td><td>%-1.1f</td><td>%-2.2f</td><td>%-2.2f\n",
                                (float)conn_bytes / KBYTE, (float) my_bytes / MBYTE,
                                (float)bytes / MBYTE);
-                    
+
                     if (ws_record->status == SERVER_BUSY_READ)
                         ap_rprintf(r,
                                    "</td><td>?</td><td nowrap>?</td><td nowrap>..reading.. </td></tr>\n\n");
@@ -792,11 +802,11 @@ static int status_handler(request_rec *r)
 
     {
         /* Run extension hooks to insert extra content. */
-        int flags = 
-            (short_report ? AP_STATUS_SHORT : 0) | 
+        int flags =
+            (short_report ? AP_STATUS_SHORT : 0) |
             (no_table_report ? AP_STATUS_NOTABLE : 0) |
             (ap_extended_status ? AP_STATUS_EXTENDED : 0);
-        
+
         ap_run_status_hook(r, flags);
     }
 
