@@ -1,55 +1,17 @@
-/* ====================================================================
- * The Apache Software License, Version 1.1
+/* Copyright 2000-2005 The Apache Software Foundation or its licensors, as
+ * applicable.
  *
- * Copyright (c) 2000-2003 The Apache Software Foundation.  All rights
- * reserved.
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
  *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
+ *     http://www.apache.org/licenses/LICENSE-2.0
  *
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- *
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in
- *    the documentation and/or other materials provided with the
- *    distribution.
- *
- * 3. The end-user documentation included with the redistribution,
- *    if any, must include the following acknowledgment:
- *       "This product includes software developed by the
- *        Apache Software Foundation (http://www.apache.org/)."
- *    Alternately, this acknowledgment may appear in the software itself,
- *    if and wherever such third-party acknowledgments normally appear.
- *
- * 4. The names "Apache" and "Apache Software Foundation" must
- *    not be used to endorse or promote products derived from this
- *    software without prior written permission. For written
- *    permission, please contact apache@apache.org.
- *
- * 5. Products derived from this software may not be called "Apache",
- *    nor may "Apache" appear in their name, without prior written
- *    permission of the Apache Software Foundation.
- *
- * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
- * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
- * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
- * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
- * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
- * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
- * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
- * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
- * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
- * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- * ====================================================================
- *
- * This software consists of voluntary contributions made by many
- * individuals on behalf of the Apache Software Foundation.  For more
- * information on the Apache Software Foundation, please see
- * <http://www.apache.org/>.
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
  */
 
 #include <stdio.h>
@@ -57,6 +19,10 @@
 #include "apr_reslist.h"
 #include "apr_thread_proc.h"
 
+#if APR_HAVE_TIME_H
+#include <time.h>
+#endif /* APR_HAVE_TIME_H */
+
 #if !APR_HAS_THREADS
 
 int main(void)
@@ -70,12 +36,12 @@ int main(void)
 #define RESLIST_MIN   3
 #define RESLIST_SMAX 10
 #define RESLIST_HMAX 20
-#define RESLIST_TTL  (350000LL) /* 35 ms */
+#define RESLIST_TTL  APR_TIME_C(350000) /* 35 ms */
 #define CONSUMER_THREADS 25
 #define CONSUMER_ITERATIONS 250
-#define CONSTRUCT_SLEEP_TIME  (250000LL) /* 25 ms */
-#define DESTRUCT_SLEEP_TIME   (100000LL) /* 10 ms */
-#define WORK_DELAY_SLEEP_TIME (150000LL) /* 15 ms */
+#define CONSTRUCT_SLEEP_TIME  APR_TIME_C(250000) /* 25 ms */
+#define DESTRUCT_SLEEP_TIME   APR_TIME_C(100000) /* 10 ms */
+#define WORK_DELAY_SLEEP_TIME APR_TIME_C(150000) /* 15 ms */
 
 typedef struct {
     apr_interval_time_t sleep_upon_construct;
@@ -98,7 +64,8 @@ static apr_status_t my_constructor(void **resource, void *params,
     res = apr_palloc(pool, sizeof(*res));
     res->id = my_params->c_count++;
 
-    printf("++ constructing new resource [id:%d]\n", res->id);
+    printf("++ constructing new resource [id:%d, #%d/%d]\n", res->id,
+       my_params->c_count, my_params->d_count);
 
     /* Sleep for awhile, to simulate construction overhead. */
     apr_sleep(my_params->sleep_upon_construct);
@@ -114,8 +81,8 @@ static apr_status_t my_destructor(void *resource, void *params,
     my_resource_t *res = resource;
     my_parameters_t *my_params = params;
 
-    printf("-- destructing old resource [id:%d, #%d]\n", res->id,
-           my_params->d_count++);
+    printf("-- destructing old resource [id:%d, #%d/%d]\n", res->id,
+           my_params->c_count, ++my_params->d_count);
 
     apr_sleep(my_params->sleep_upon_destruct);
 
@@ -138,26 +105,140 @@ static void * APR_THREAD_FUNC resource_consuming_thread(apr_thread_t *thd,
 
     for (i = 0; i < CONSUMER_ITERATIONS; i++) {
         my_resource_t *res;
-        rv = apr_reslist_acquire(rl, (void**)&res);
+        void *vp;
+        rv = apr_reslist_acquire(rl, &vp);
         if (rv != APR_SUCCESS) {
             fprintf(stderr, "Failed to retrieve resource from reslist\n");
             apr_thread_exit(thd, rv);
             return NULL;
         }
+        res = vp;
         printf("  [tid:%d,iter:%d] using resource id:%d\n", thread_info->tid,
                i, res->id);
         apr_sleep(thread_info->work_delay_sleep);
-        rv = apr_reslist_release(rl, res);
-        if (rv != APR_SUCCESS) {
-            fprintf(stderr, "Failed to return resource to reslist\n");
-            apr_thread_exit(thd, rv);
-            return NULL;
-        }
+/* simulate a 5% chance of the resource being bad */
+        if ( drand48() < 0.95 ) {
+           rv = apr_reslist_release(rl, res);
+            if (rv != APR_SUCCESS) {
+                fprintf(stderr, "Failed to return resource to reslist\n");
+                apr_thread_exit(thd, rv);
+                return NULL;
+            }
+       } else {
+           printf("invalidating resource id:%d\n", res->id) ;
+           rv = apr_reslist_invalidate(rl, res);
+            if (rv != APR_SUCCESS) {
+                fprintf(stderr, "Failed to invalidate resource\n");
+                apr_thread_exit(thd, rv);
+                return NULL;
+            }
+       }
     }
 
     return APR_SUCCESS;
 }
 
+static void test_timeout(apr_reslist_t *rl)
+{
+    apr_status_t rv;
+    my_resource_t *resources[RESLIST_HMAX];
+    my_resource_t *res;
+    void *vp;
+    int i;
+
+    printf("Setting timeout to 1000us: ");
+    apr_reslist_timeout_set(rl, 1000);
+    fprintf(stdout, "OK\n");
+
+    /* deplete all possible resources from the resource list 
+     * so that the next call will block until timeout is reached 
+     * (since there are no other threads to make a resource 
+     * available)
+     */
+
+    for (i = 0; i < RESLIST_HMAX; i++) {
+        rv = apr_reslist_acquire(rl, (void**)&resources[i]);
+        if (rv != APR_SUCCESS) {
+            fprintf(stderr, "couldn't acquire resource: %d\n", rv);
+            exit(1);
+        }
+    }
+
+    /* next call will block until timeout is reached */
+    rv = apr_reslist_acquire(rl, &vp);
+    if (!APR_STATUS_IS_TIMEUP(rv)) {
+        fprintf(stderr, "apr_reslist_acquire()->%d instead of TIMEUP\n", 
+                rv);
+        exit(1);
+    }
+    res = vp;
+
+    /* release the resources; otherwise the destroy operation
+     * will blow
+     */
+    for (i = 0; i < RESLIST_HMAX; i++) {
+        rv = apr_reslist_release(rl, resources[i]);
+        if (rv != APR_SUCCESS) {
+            fprintf(stderr, "couldn't release resource: %d\n", rv);
+            exit(1);
+        }
+    }
+}
+
+static void test_shrinking(apr_reslist_t *rl)
+{
+    apr_status_t rv;
+    my_resource_t *resources[RESLIST_HMAX];
+    my_resource_t *res;
+    int i;
+    int sleep_time = RESLIST_TTL / RESLIST_HMAX;
+
+    /* deplete all possible resources from the resource list */
+    for (i = 0; i < RESLIST_HMAX; i++) {
+        rv = apr_reslist_acquire(rl, (void**)&resources[i]);
+        if (rv != APR_SUCCESS) {
+            fprintf(stderr, "couldn't acquire resource: %d\n", rv);
+            exit(1);
+        }
+    }
+
+    /* Free all resources above RESLIST_SMAX - 1 */
+    for (i = RESLIST_SMAX - 1; i < RESLIST_HMAX; i++) {
+        rv = apr_reslist_release(rl, resources[i]);
+        if (rv != APR_SUCCESS) {
+            fprintf(stderr, "couldn't release resource: %d\n", rv);
+            exit(1);
+        }
+    }
+
+    for (i = 0; i < RESLIST_HMAX; i++) {
+        rv = apr_reslist_acquire(rl, (void**)&res);
+        if (rv != APR_SUCCESS) {
+            fprintf(stderr, "couldn't acquire resource: %d\n", rv);
+            exit(1);
+        }
+        apr_sleep(sleep_time);
+        rv = apr_reslist_release(rl, res);
+        if (rv != APR_SUCCESS) {
+            fprintf(stderr, "couldn't release resource: %d\n", rv);
+            exit(1);
+        }
+    }
+    apr_sleep(sleep_time);
+
+    /*
+     * Now free the remaining elements. This should trigger the shrinking of
+     * the list
+     */
+    for (i = 0; i < RESLIST_SMAX - 1; i++) {
+        rv = apr_reslist_release(rl, resources[i]);
+        if (rv != APR_SUCCESS) {
+            fprintf(stderr, "couldn't release resource: %d\n", rv);
+            exit(1);
+        }
+    }
+}
+
 static apr_status_t test_reslist(apr_pool_t *parpool)
 {
     apr_status_t rv;
@@ -167,6 +248,7 @@ static apr_status_t test_reslist(apr_pool_t *parpool)
     int i;
     apr_thread_t *my_threads[CONSUMER_THREADS];
     my_thread_info_t my_thread_info[CONSUMER_THREADS];
+    srand48(time(0)) ;
 
     printf("Creating child pool.......................");
     rv = apr_pool_create(&pool, parpool);
@@ -224,6 +306,17 @@ static apr_status_t test_reslist(apr_pool_t *parpool)
     }
     printf("\nDone!\n");
 
+    test_timeout(rl);
+
+    test_shrinking(rl);
+    if (params->c_count - params->d_count != RESLIST_SMAX) {
+        printf("FAILED: Resourcelist has not shrinked back to RESLIST_SMAX\n");
+        return APR_EGENERAL;
+    }
+    else {
+        printf("OK: Resource list shrinked back to RESLIST_SMAX\n");
+    }
+
     printf("Destroying resource list.................");
     rv = apr_reslist_destroy(rl);
     if (rv != APR_SUCCESS) {
