@@ -17,11 +17,11 @@
 /*
  * Simple program to rotate Apache logs without having to kill the server.
  *
- * Contributed by Ben Laurie <ben@algroup.co.uk>
+ * Contributed by Ben Laurie <ben algroup.co.uk>
  *
  * 12 Mar 1996
  *
- * Ported to APR by Mladen Turk <mturk@mappingsoft.com>
+ * Ported to APR by Mladen Turk <mturk mappingsoft.com>
  *
  * 23 Sep 2001
  *
@@ -54,7 +54,7 @@
 #endif
 
 #define BUFSIZE         65536
-#define ERRMSGSZ        82
+#define ERRMSGSZ        128
 
 #ifndef MAX_PATH
 #define MAX_PATH        1024
@@ -171,7 +171,7 @@ int main (int argc, const char * const argv[])
             apr_finfo_t finfo;
             apr_off_t current_size = -1;
 
-            if ((nLogFD != NULL) && 
+            if ((nLogFD != NULL) &&
                 (apr_file_info_get(&finfo, APR_FINFO_SIZE, nLogFD) == APR_SUCCESS)) {
                 current_size = finfo.size;
             }
@@ -188,7 +188,8 @@ int main (int argc, const char * const argv[])
 
         if (nLogFD == NULL) {
             int tLogStart;
-                
+            apr_status_t rv;
+
             if (tRotation) {
                 tLogStart = (now / tRotation) * tRotation;
             }
@@ -208,22 +209,28 @@ int main (int argc, const char * const argv[])
                 sprintf(buf2, "%s.%010d", szLogRoot, tLogStart);
             }
             tLogEnd = tLogStart + tRotation;
-            apr_file_open(&nLogFD, buf2, APR_READ | APR_WRITE | APR_CREATE | APR_APPEND,
-                          APR_OS_DEFAULT, pool);
-            if (nLogFD == NULL) {
+            rv = apr_file_open(&nLogFD, buf2, APR_WRITE | APR_CREATE | APR_APPEND,
+                               APR_OS_DEFAULT, pool);
+            if (rv != APR_SUCCESS) {
+                char error[120];
+
+                apr_strerror(rv, error, sizeof error);
+
                 /* Uh-oh. Failed to open the new log file. Try to clear
                  * the previous log file, note the lost log entries,
                  * and keep on truckin'. */
                 if (nLogFDprev == NULL) {
-                    fprintf(stderr, "1 Previous file handle doesn't exists %s\n", buf2);
+                    fprintf(stderr, "Could not open log file '%s' (%s)\n", buf2, error);
                     exit(2);
                 }
                 else {
                     nLogFD = nLogFDprev;
-                    sprintf(errbuf,
-                            "Resetting log file due to error opening "
-                            "new log file. %10d messages lost.\n",
-                            nMessCount);
+                    /* Try to keep this error message constant length
+                     * in case it occurs several times. */
+                    apr_snprintf(errbuf, sizeof errbuf,
+                                 "Resetting log file due to error opening "
+                                 "new log file, %10d messages lost: %-25.25s\n",
+                                 nMessCount, error);
                     nWrite = strlen(errbuf);
                     apr_file_trunc(nLogFD, 0);
                     if (apr_file_write(nLogFD, errbuf, &nWrite) != APR_SUCCESS) {
